#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <vector>

#include <cstring>
#include <cctype>
#include <cassert>
#include <cstdio>
using namespace std;

#include "vbible.h"

string IntToStr (int x) {
    char buf [10];
    sprintf (buf, "%d", x);
    string result = buf;
    return result;
}

//CVerse class implementation

CVerse::CVerse (char* t) : text(t) {
    if (t) {
        text = new char [strlen(t) + 1];
        strcpy (text, t);
    }
}
  
CVerse::CVerse (istream & f) {
    load (f);
}
  
void CVerse::load (istream & f) {
    char buf [1000];
    unsigned len = (unsigned) ((unsigned) f.get() | (unsigned) f.get() << 8);
    f.read (buf, len);
    buf [len] = '\0';
    text = new char [len+1];
    strcpy (text, buf);
};
  
CVerse::operator const char * () const {
    return text;
}

const char * CVerse::getText () const {
    return text;
}

CVerse::~CVerse () {
    if (text)
    	delete [] text;
}

//CVerse class implementation
CChapter::CChapter (unsigned char count, bool to_load, istream& f) 
    : verses(count)
{
    for (int i = 0; i < count; i++) 
    	verses [i] = NULL;
    if (to_load)
    	load (f);
}
  
void CChapter::load (istream& f) {
    for (size_t i = 0; i < verses.size (); i++)
    	verses [i] = new CVerse (f);
}

size_t CChapter::size() const { return verses.size (); }

const CVerse& CChapter::operator [] (size_t i) const {
    if (i > verses.size())
    	throw ENoSuchVerse();
    return *verses[i];
}
  
// CVerse& CChapter::operator [] (size_t i) {
//     if (i > verses.size())
//     	throw ENoSuchVerse();
//     return *verses[i];
// }
    
CChapter::~CChapter () {
    for (size_t i = 0; i < verses.size (); i++)
    	if (verses[i])
        	delete verses[i];
}
   
//CBook class implementation  
CBook::CBook (unsigned char count) 
    : chapters (count)
{
    for (int i = 0; i < count; i++) 
    	chapters [i] = NULL;
}

void CBook::add (CChapter* chap) {
    chapters.push_back (chap);
}

size_t CBook::size () const { return chapters.size (); }
  
const CChapter & CBook::operator [] (size_t i) const {
    if (i > chapters.size())
    	throw ENoSuchChapter();
    return *chapters[i];
}
  
// CChapter & CBook::operator [] (size_t i) {
//     if (i > chapters.size())
//     	throw ENoSuchChapter();
//     return *chapters[i];
// }
 
CBook::~CBook () {
    for (size_t i = 0; i < chapters.size (); i++)
    	if (chapters[i])
        	delete chapters[i];
}

//CRef class implementation  
CRef::CRef ()
    : error (true)
{
}


CRef::CRef (const char* whichbook, int whichchap, int whichverse, int cnt) 
{
    set (whichbook, whichchap, whichverse, cnt);
}

CRef::CRef (int id, int whichchap, int whichverse, int cnt) 
{
    set (id, whichchap, whichverse, cnt);
}

CRef::operator bool () const {
    return ! error;
}
                                                  
string CRef::toString () const { 
	if (error)
    	return "(error)";
    
    string result = book + " " + IntToStr (chapter) + ":" + IntToStr(verse);
    if (count > 1)
    	result += "-" + IntToStr (verse + count - 1);
    return result;
}


void CRef::set (const char* whichbook, int whichchap, int whichverse, int cnt)
{
    bookNum = -1;
    book = whichbook;
    chapter = whichchap;
    verse = whichverse;
    count = cnt;

    //Let's determine books number
    error = ! getBookNum (whichbook, bookNum);
}

void CRef::set (int id, int whichchap, int whichverse, int cnt) {
    bookNum = id;
    chapter = whichchap;
    verse = whichverse;
    count = cnt;

    //Let's determine books name
    error = ! getBookName (bookNum, book);
}

bool CRef::getBookNum (const char* whichbook, int & bookNum) {
    ifstream books ("booknames.ref");
    char bk [20];

    int bk_num;
    for (bk_num = 1; bk_num <= 66; bk_num++) {
        books.getline (bk, 20);
        if (! strcmp (bk, whichbook))
            break;
    }
    books.close ();

    if (bk_num > 66) {
    	return false;
    }
    bookNum = bk_num;
    return true;
}

bool CRef::getBookName (int id, string & name) {
    ifstream books ("booknames.ref");
    if (id < 1 || id > 66)
	    return false;
    char bk [20];
    for (int i = 1; i < id; i++)
    	books.getline (bk, 20);
    books.getline (bk, 20);
    name = bk;
    return true;
}


//CBible class implementation
CBible::CBible (const char * filename, IProgressIndicator * progress) : books(66) {
    for (size_t i = 0; i < books.size (); i++)
        books [i] = NULL;

	ifstream f (filename, ios::binary);
	ifstream ref ("bible.ref");

	if (! f)
		throw EBible ("\n      !\n");

	long fsize = f.seekg (0, ios::end).tellg ();
	f.seekg (0, ios::beg);

	int completed = 0; //percentage
	for (int bk_num = 0; bk_num < 66; bk_num++) {
		int chaps;
		ref >> chaps;
		books [bk_num] = new CBook;

		while (chaps--) {
			int vs;
			ref >> vs;
			CChapter * chap = new CChapter (vs, true, f);
			books [bk_num] -> add (chap);

			if (progress != NULL && f.tellg() >= (completed + 1) * (fsize / 99)) {
				progress->setProgress(completed);
				completed++;
			}
		}
	}
	ref.close ();
	f.close ();
}

const CBook & CBible::operator [] (int index) const {
    return *books[index];
}

// CBook & CBible::operator [] (int index) {
//     return *books[index];
// }
  
CBible::~CBible () {
    for (size_t i = 0; i < books.size (); i++)
    if (books[i])
        delete books[i];
}

string CBible::getPassage (const CRef& ref, bool add_verse_number) const {
    stringstream result;
    for (int i = 0; i < ref.count; i++) {
        CBook& b = *books [ref.bookNum - 1];
        if (add_verse_number)
            result << ref.verse + i << " ";
        result << b [ref.chapter - 1][ref.verse - 1 + i];
        result << endl;
        if (i < ref.count - 1)
            result << endl;     //if not the last verse, put a new line befor the next one
    }
    result << ends;
    return result.str().c_str();
}


void CBible::search (const char *text, vector<CRef> & result, bool whole_word) const {
    result.clear ();
    for (int b = 0; b < 66; b++) {
        CBook & book = *books [b];
        for (size_t ch = 0; ch < book.size (); ch++) {
            for (size_t v = 0; v < book [ch].size (); v++) {
                const char * verse = book [ch][v];
                const char * found = strstr (verse, text);
                
                if (found && whole_word) { //if we search for whole words only
                    if (found != verse && iscyrillic (*(found - 1)))
                    //if not the first char and the previous is cyr
                    continue;
                    found += strlen (text); //move after the end of the searched word
                    if (iscyrillic (*found))  //and if the char there is cyrillic (> 127)
                    continue;     //the word continues, so we skip this match
                }
                
                if (found)
                    result.push_back (CRef (b + 1, ch + 1, v + 1));
            }
        }
    }
}
