/***************************************************************************
                          vbible.h  -  description
                             -------------------
    begin                :   1 2004
    copyright            : (C) 2004 by Radoslav Atanasov
    email                : rsdevelop@mail.bg
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef VBible_H
#define VBible_H

#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <vector>

#include <cstring>
#include <cctype>
#include <cstdio>
using namespace std;

inline bool iscyrillic (char ch) {
    return ((unsigned char)ch) > 0x7F;
}

#ifdef DEBUG
  #define DUMP(x) cerr << #x "=" << (x) << endl;
#else
  #define DUMP(x)
#endif

string IntToStr (int x);

class EBible {
 	string message;
public:
  	EBible (const char * msg = NULL) : message (msg) {}
	string what () const { return message; }
};

class ENoSuchVerse : public EBible {};
class ENoSuchChapter : public EBible {};

class CVerse {
protected:
    char *text;
public:
    CVerse (char* t = NULL);
    CVerse (istream & f);
    void load (istream & f);
    operator const char * () const;
    const char * getText () const;
    ~CVerse ();
};

class CChapter {
protected:
    vector<CVerse*> verses;
public:
    CChapter (unsigned char count, bool to_load = false, istream& f = cin);
    void load (istream& f);
    size_t size() const;
    const CVerse& operator [] (size_t i) const;
    //CVerse& operator [] (size_t i);
    ~CChapter ();
};

class CBook {
protected:
    vector<CChapter*> chapters;
public:
    CBook (unsigned char count = 0);
    void add (CChapter* chap);
    size_t size () const;
    const CChapter & operator [] (size_t i) const;
    //CChapter & operator [] (size_t i);
    ~CBook ();
};

class CRef {
    bool error;
public:
    string book;
    int chapter, verse, count;
    int bookNum;
    
    CRef ();
    CRef (const char* whichbook, int whichchap, int whichverse, int cnt = 1);
    CRef (int id, int whichchap, int whichverse, int cnt = 1);
    void set (const char* whichbook, int whichchap, int whichverse, int cnt = 1);
    void set (int id, int whichchap, int whichverse, int cnt = 1);
    operator bool () const;
    string toString () const;
    
    static bool getBookNum (const char* whichbook, int & bookNum); //false on error
    static bool getBookName(int id, string & name);
};

class IProgressIndicator {
public:
	virtual void setProgress (int p) = 0;
};

class CBible {
    vector<CBook*> books;
public:
    CBible (const char * filename, IProgressIndicator * progress = NULL);
    ~CBible ();
    
    string getPassage (const CRef& ref, bool add_verse_number = true) const;
    void search (const char *text, vector<CRef> & result, bool whole_word = false) const;
    const CBook & operator [] (int index) const;
    //CBook & operator [] (int index);
};

#endif
